// IDA HELPER TOOL
//
//
// - Set prototype functions based no mangled symbols
//		custom types should be exist on IDA before running this script
//
//

#include <idc.idc>

#define VERBOSE 1

#define VAC 1

static main(void)
{
	ProcFunctions(".text");
	ProcFunctions(".plt");
	ProcFunctions(".data");
}

static ProcFunctions(segname)
{
	auto start, end, ea;
	auto i, ret;
	auto ord, name;
	auto ok, err, errtype;

	start = SegStart(GetSeg(segname));
	end = SegEnd(start);
	ea = start - 1;
	while ((ea = NextFunction(ea)) && ea >= start && ea <= end)
	{
		name = GetFunctionName(ea);

		if(strlen(name) > 0 && strlen(GetType(ea)) > 5)
		{
				Message("%s|o|%s|%s\n", name, Demangle(name, 0), GetType(ea));
		}
	}
}

static GetSeg(segname)
{
	auto ea;
	ea = 0;
	while((ea = NextSeg(ea)))
	{
		if(SegName(ea) == segname)
			return ea;
	}
	return 0;
}

static GetClass(symbol)
{
	auto i, myclass;

	for(i = 0; i < strlen(symbol); i++)
	{
		if(symbol[i] == ":" && symbol[i+1] == ":")
		{
			myclass = substr(symbol, 0, i);
			return myclass;
		}
	}
	return myclass;
}

static FixClassDec(symbol, ord)
{
	auto i, myclass, argpos, newsym;
	
	for(i = 0; i < (strlen(symbol) - 1); i++)
	{
		if(symbol[i] == ":" && symbol[i+1] == ":")
		{
			myclass = GetClass(symbol);
			symbol[i] = "_";
			symbol[i+1] = "_";
		}
		

		if(symbol[i] == "(")
		{
			argpos = i+1;
		}
		
		if(symbol[i] == ")")
		{
			symbol[i+1] = ";";
		}
	}
	
//	if(argpos && strlen(myclass) > 1 && InVTBL(ord))
	if(argpos && strlen(myclass) > 1)
	{
		newsym = substr(symbol, 0, argpos-1) +"(" + myclass + " *this, " + substr(symbol, argpos, 255);
	 	symbol = newsym;
	}
	return FixVoid(symbol);
}

static FixVoid(symbol)
{
	auto i;

	while((i = strstr(symbol,  ", void")) > 1)
	{
		symbol[i] = " ";
		symbol[i+1] = " ";
		symbol[i+2] = " ";
		symbol[i+3] = " ";
		symbol[i+4] = " ";
		symbol[i+5] = " ";
	}
	
	while((i = strstr(symbol,  ";onst")) > 1)
	{
		symbol[i+1] = " ";
		symbol[i+2] = " ";
		symbol[i+3] = " ";
		symbol[i+4] = " ";
		symbol[i] = " ";
	}
	return symbol;
}

static InVTBL(obj)
{
	auto xea, plt;
	
	for(xea=DfirstB(obj); xea != BADADDR; xea=DnextB(obj, xea))
	{
		if(strstr(Demangle(Name(xea), 0), "vtbl") >= 0)
		{
			return 1;
		}
	}
	return 0;
}